-- load in the OpenGL module
local gl = require("opengl")
local GL = gl

initialized = false

Programs = {}

-- 
function create3DTexture(width, height, depth, numComponents)
	tex = jit.new("jit.gl.texture")
	if depth > 1 then tex.dim = {width, height, depth}
		else tex.dim = {width, height}
	end
	tex.wrap = {"clampedge", "clampedge", "clampedge"}
	tex.filter = "linear"
	tex.type = "half"
	if numComponents == 1 then tex.colormode = "alpha"
		elseif numComponents == 2 or numComponents == 3 then tex.colormode = "rgb" -- ? no 'rg'
		else tex.colormode = "argb"
	end
end

--
function createProgram(fileName)
	local shader = jit.new("jit.gl.shader", this.drawto)
	shader.file = fileName
	shader:bind() -- force loading and compilation
	shader:unbind()
	return shader
end

-- Draw FullScreenQuad
function fullScreenQuad()
	gl.Begin(GL.TRIANGLE_STRIP)
		gl.Vertex(-1, -1)
		gl.Vertex(1, -1)
		gl.Vertex(-1, 1)
		gl.Vertex(1, 1)
	gl.End()
end

--
function initialize()
	print("Init...")
	Programs["fillTexture3D"] = createProgram("fillTexture3D.jxs")
	Programs["texture3DViewer"] = createProgram("texture3Dviewer.jxs")
	create3DTexture(640,480,10,3)
	fillTexture3D()
	print("Init done !")
end

--
function fillTexture3D()
	local p = Programs["fillTexture3D"]
	p:bind()
	jit.gl.begincapture(tex.name)
	for layer = 0, 9 do
		p:param("gLayer", layer)
		fullScreenQuad()
	end
	jit.gl.endcapture(tex.name, 0)
end

--
function viewTexture3D(layer)
	local p = Programs["texture3DViewer"]
	p:bind()
	gl.Viewport(0, 0, 640, 480)
	gl.ClearColor(0, 0, 0, 0)
	jit.gl.bindtexture(tex.name, 0)
	p:param("gLayer", layer)
	fullScreenQuad()
	jit.gl.unbindtexture(tex.name, 0)
end



-- 
function draw()
	if initialized == false then
		initialize()
		initialized = true
	end
	viewTexture3D(0)
end


-- whenever drawto changes, other jit.gls must also be updated
function dest_changed()
	for key,value in pairs(Programs) do 
		value.drawto = this.drawto
	end
	tex.drawto = this.drawto	
end
